% HISTDEMO  Demonstrates the implementation of chi2 criterion application for
% checking the data set empirical distribution for normality

% (C) 2015 Alexey Voskov
% License: public domain
function histdemo
% Construct empirical histogram
N = 1000;
x = randn(1,N);
nbins = round(log(N)/log(2));
[Oi, Xbc] = hist(x, nbins);
% Construct theoretical histogram
mu = mean(x);
s = std(x);
Xbc = [-Inf Xbc Inf];
Xi = (Xbc(2:end) + Xbc(1:end-1))/2;
Ei = (normcdf(Xi(2:end),mu,s) - normcdf(Xi(1:end-1),mu,s)) * N;
% Apply chi2 (Pearson) criterion and generate the report
fprintf('Histogram construction and Pearson criterion demonstration\n');
fprintf('Number of points: %d\n', N);
fprintf('%7s %7s\n', 'Oi', 'Ei');
fprintf('%7.2f %7.2f\n', [Oi;Ei]);
chi2emp = sum((Ei - Oi).^2 ./ Ei);
chi2crit = chi2inv(0.95, nbins - 1);
fprintf('\n');
fprintf('chi2emp = %.3f\n', chi2emp);
fprintf('chi2(0.95,nbins-1) = %.3f\n', chi2crit);
% Draw the histogram
hist(x,nbins, 'FaceColor', 'Yellow', 'EdgeColor', 'red', 'LineWidth', 5);
set(gca, 'FontSize', 22);
% Create LaTeX report
msg = 'Do you want to create LaTeX report?';
title = 'Histogram demo';
strans = questdlg(msg,title,'Yes','No','Yes');
if strcmp(strans, 'Yes')
  title = ['\\documentclass[12pt]{article}\n',...
  '\\usepackage{float}\\usepackage{longtable}\n',...
  '\\usepackage[utf8]{inputenc}\\usepackage[english]{babel}\n',...
  '\\usepackage{amsmath}\\usepackage{amsfonts}\n',...
  '\\usepackage[left=2cm,right=2cm,top=2cm,bottom=2cm]{geometry}\n',...
  '\\usepackage{graphicx}\\sloppy\n',...
  '\\begin{document}\n'];
  docend = '\\end{document}';
  print('histdemo', '-dpng', '-r300'); % PNG format, 300DPI
  fd = fopen('histdemo.tex','w');
  fprintf(fd, title);
  % Type an introduction
  fprintf(fd, '\\section*{Histogram construction example}\n');
  fprintf(fd, 'This is an example of normality test implementation based on Pearson''s ($\\chi^2$) criterion.\n');
  fprintf(fd, 'A data set consists of $N=%d$ points and is generated by \\texttt{randn} function.\n', N);
  fprintf(fd, 'The number of histogram bins is $\\log_2{N}=%d$.\n\n', nbins);
  fprintf(fd, 'The first step is the estimation of mathematical expectation $\\mu$ and variance $\\sigma$\n');
  fprintf(fd, 'to build theoretical normal distribution. The next formula are used:\n\n');
  fprintf(fd, '$$\n\\mu = \\frac{1}{N}\\sum_iX_i;~');
  fprintf(fd, '\\sigma=\\sqrt{\\frac{\\sum_i\\left(X_i - \\mu\\right)^2}{N-1}}\n$$\n\n');
  fprintf(fd, 'For the input data set $\\mu=%.4f$, $\\sigma=%.4f$\n\n', mu, s);
  fprintf(fd, 'The second step is the construction of empirical histogram (that gives us observed frequencies $O_i$) ');
  fprintf(fd, 'and calculation of theoretical histogram using $\\mu$ and $\\sigma$ values obtained above\n');
  fprintf(fd, 'The next formula is used for $E_i$ (expected frequencies calculation):');
  fprintf(fd, '\n\n$$\nE_i = N\\int_{x_{i,\\textrm{min}}}^{{x_{i,\\textrm{max}}}}');
  fprintf(fd, '  \\frac{1}{\\sqrt{2\\pi}\\sigma}');
  fprintf(fd, '  \\exp{\\left(-\\frac{\\left(x-\\mu\\right)^2}{2\\sigma^2}\\right)}dx\n');
  fprintf(fd, '$$\n\n');
  fprintf(fd, 'Obtained $E_i$ and $O_i$ values are given in Table~\\ref{tbl:hist}, ');
  fprintf(fd, 'empirical histogram is shown at Figure~\\ref{fig:hist}\n\n');
  fprintf(fd, 'The third step is applying of Pearson''s $\\chi^2$ test using the next relations:\n');
  fprintf(fd, '\n\n$$\\chi^2_\\textrm{emp} = \\sum_i\\frac{(O_i-E_i)^2}{E_i} = %.3f\n$$\n\n', chi2emp);
  fprintf(fd, '\n\n$$\\chi^2_{\\alpha,f}\\left(\\alpha=0.95,f=n_\\textrm{bins}-1=%d\\right) = %.3f\n$$\n\n', ...
    nbins - 1, chi2crit);
  % Show the results of chi2 test
  if chi2emp <= chi2crit
    fprintf(fd, '$\\chi^2_\\textrm{emp} \\le \\chi^2_\\textrm{crit}$ and the empirical distribution can be considered normal.\n\n');    
  else
    fprintf(fd, '$\\chi^2_\\textrm{emp} > \\chi^2_\\textrm{crit}$ and the empirical distribution cannot be considered normal.\n\n');  
  end
  % Type the table with histogram frequences
  fprintf(fd, '\\begin{table}[H]\\caption{Observed and expected histogram frequencies}\n');
  fprintf(fd, '\\begin{center}\\begin{tabular}{|c|c|c|c|c|}\n');
  fprintf(fd, '\\hline\n$x_{i,\\textrm{min}}$ & $x_{i,\\textrm{max}}$ & $O_i$ & $E_i$ & $(O_i-E_i)^2/E_i$ \\\\\n\\hline\n');
  for i=1:numel(Ei)
    fprintf(fd, '%s & %s & %.1f & %.1f & %.3f \\\\\n', ...    
      Xi_to_str(Xi(i)), Xi_to_str(Xi(i+1)), Oi(i), Ei(i), (Oi(i)-Ei(i)).^2./Ei(i));
  end
  fprintf(fd, '\\hline\n');
  fprintf(fd, '\\end{tabular}\\end{center}\\label{tbl:hist}\\end{table}\n');  
  % Insert the figure with empirical histogram
  fprintf(fd, '\\begin{figure}[H]\n');
  fprintf(fd, '\\center{\\includegraphics[width=12cm]{histdemo}}\n');
  fprintf(fd, '\\caption{Empirical histogram}\\label{fig:hist}\n');
  fprintf(fd, '\\end{figure}\n');
  % Insert the table with the data set
  xf = x;
  while mod(numel(xf), 8) > 0
    xf = [xf, NaN];
  end
  xf = reshape(xf, 8, numel(xf) / 8);
  hdrtxt = ['\\hline\n',...
    '$X_i$ & $X_i$ & $X_i$ & $X_i$ & $X_i$ & $X_i$ & $X_i$ & $X_i$ \\\\\n',...
    '\\hline\n'];  
  fprintf(fd, '\n\n\\section*{Input data set}\n\n');
  fprintf(fd, '\n\n\\begin{center}\\begin{longtable}[H]{|c|c|c|c|c|c|c|c|}\n');
  fprintf(fd, '\\caption{Input data set}\\\\\n');
  fprintf(fd, hdrtxt);
  fprintf(fd, '\\endfirsthead\n');
  fprintf(fd, hdrtxt);  
  fprintf(fd, '\\endhead\n');
  fprintf(fd, hdrtxt);
  fprintf(fd, '\\endfoot\n');
  fprintf(fd, hdrtxt);
  fprintf(fd, '\\endlastfoot\n');
  str = sprintf('%.4f & %.4f & %.4f & %.4f & %.4f & %.4f & %.4f & %.4f    \\\\\n', xf);  
  fprintf(fd, '%s\n', str(1:end-5));
  fprintf(fd, '\\end{longtable}\\end{center}\n');
  % Finalize the document and close the output .tex file
  fprintf(fd, docend);  
  fclose(fd);
end
end

function str = Xi_to_str(Xi)
if isinf(Xi) && Xi < 0
  str = '$-\infty$';
elseif isinf(Xi) && Xi > 0
  str = '$+\infty$';
else
  str = sprintf('$%.3f$', Xi);
end
end