% MONTECARLO_3D  An example of geometric Monte-Carlo algoritm implementation.
% The volume of cone with circular base is integrated in this script.
% Usage:
%   [Vmc, Van] = montecarlo_3d
%   [Vmc, Van] = montecarlo_3d(r, h)
% Inputs:
%   r -- radius of the circular base of the cone (must be scalar)
%   h -- height of the cone (must be scalar)
% Outputs:
%   Vmc -- cone volume estimated by the Monte-Carlo method
%   Van -- cone volume estimated by the analytical formula
% 
% This script is compatible with GNU Octave and MATLAB

% (C) 2015 Alexey Voskov
% License: public domain
function [Vmc, Van] = montecarlo_3d(r, h)
% Input arguments control
if nargin == 0
  h = 1; % Default height
  r = 1; % Default radius of the circular base
elseif nargin ~= 2
  error('Wrong number of arguments');
end
if ~isscalar(r) || ~isreal(r) || ~isnumeric(r) || r <= 0
  error('r must be a real scalar positive double');
end
if ~isscalar(h) || ~isreal(h) || ~isnumeric(h) || h <= 0
  error('h must be a real scalar positive double');
end
% Parameters of the numeric integration
N = 100000; % Number of points
xmin = -1.1*r; ymin = xmin;
xmax =  1.1*r; ymax = xmax;
zmin = 0; zmax = 1.1*h;
% Geometric Monte-Carlo algorithm implementation
% a) initial random set of points
x = xmin + (xmax - xmin) * rand(1,N);
y = ymin + (ymax - ymin) * rand(1,N);
z = zmin + (zmax - zmin) * rand(1,N);
% b) select the points that are inside the cone and estimate the volume
ii = (x.^2 + y.^2 <= (r*z./h).^2) & (0 <= z) & (z <= h);
Vmc = sum(ii)/numel(ii)*(xmax-xmin)*(ymax-ymin)*(zmax-zmin);
% Show the results
Van = 1/3*pi*r^2*h;
fprintf('Monte-Carlo cone volume calculation results\n');
fprintf('h = %.4f, r = %.4f\n', h, r);
fprintf('V(mc) = %.5f (Monte-Carlo algorithm)\n', Vmc);
fprintf('V(an) = %.5f (Analytical) \n', Van);
fprintf('Relative error: %.5e\n', abs(Vmc-Van)./Van);
close all;
plot3(x(ii),y(ii),z(ii),'b.');
xlabel('x'); ylabel('y'); zlabel('z');
box on;
end